;               visual player for NES v.1b (not tested on a real NES..)
;       (c)1999 Membler Industries
;                       by Joey Parsell (5010.0951@tcon.net)
;
;       You may reuse the code and graphics that are included in this file
;       only if you agree to the following conditions..
;
;       1.  Must be freeware (duh)
;       2.  You must have fun
;       3.  Modified versions must be distributed with credits and
;           copyright info for the music (if available) and graphics program

;       If you like it or have any ideas for features, send me an email.

;       This code compiles with the CC65 devkit
;       That, and other assorted crap can be found at http://nesdev.parodius.com

.macro  inc16   addr
.local  skip
                inc addr
                bne skip
                inc addr + 1
        skip:
.endmacro

init = $8003            ; init and play routines from the music
play = $8125            ; extracted from the .NSF header

.segment "VECTORS"
        .word NMI
        .word RESET
        .word IRQ

.segment "DATA"
TempLo = $FE
TempHi = $FF
joy1 = $FD
joy1old = $FC
song = $50

a_button = 128          ; joystick stuff
b_button = 64
select_button = 32
start_button = 16
up_button = 8
down_button = 4
left_button = 2
right_button = 1


.segment "CODE"
        .incbin "silversu.rom"  ; this includes the music/replay code at $8000 (the load address specified in the .NSF header)
RESET:
        sei
        cld
        ldx #0
        stx $2000
        stx $2001
        dex
        txs
@wait:
        lda $2002
        bpl @wait
@wait2:
        lda $2002
        bpl @wait2

        ldy #0
        lda #0
clear_ram:        
        sta $0, y
        sta $200, y
        sta $300, y
        sta $400, y
        sta $500, y
        sta $600, y
        sta $700, y
        iny
        bne clear_ram

        lda #0
        sta $2006
        sta $2006
        sta TempLo
        sta TempHi
lp:
        lda #0
        sta $2007
        inc16 TempLo
        lda TempHi
        cmp #$3F
        bne lp

        lda # < font_gfx
        sta TempLo
        lda # > font_gfx
        sta TempHi
        ldy #0
        
        lda #$03
        sta $2006
        lda #$00
        sta $2006
font_load:
        lda (TempLo), y
        cmp #$7D
        beq end_load
        sta $2007
        inc16 TempLo
        jmp font_load

end_load:

        lda #$22
        sta $2006
        lda #$80
        sta $2006
        ldy #0
@load:
        lda game, y
        beq @end
        sta $2007
        iny
        jmp @load
@end:
        lda #$22
        sta $2006
        lda #$A0
        sta $2006
        ldy #0
@load2:
        lda author, y
        beq @end2
        sta $2007
        iny
        jmp @load2
@end2:
        lda #$22
        sta $2006
        lda #$C0
        sta $2006
        ldy #0
@load3:
        lda copy, y
        beq @end3
        sta $2007
        iny
        jmp @load3
@end3:
        lda #$23
        sta $2006
        lda #$20
        sta $2006
        ldy #0
@load4:
        lda info, y
        beq @end4
        sta $2007
        iny
        jmp @load4
@end4:
        lda #$23
        sta $2006
        lda #$40
        sta $2006
        ldy #0
@load5:
        lda info2, y
        beq @end5
        sta $2007
        iny
        jmp @load5
@end5:
        lda #$23
        sta $2006
        lda #$60
        sta $2006
        ldy #0
@load6:
        lda press, y
        beq @end6
        sta $2007
        iny
        jmp @load6
@end6:
     
        jsr vis_init
        lda #0
        jsr init

        lda #%10001000
        sta $2000
        lda #%00111110
        sta $2001

loop:
        lda $2002
        bpl loop
@loop:  lda $2002
        bmi @loop

        lda #start_button
        and joy1
        beq @next
        and joy1old
        bne @next
        inc song
        lda song
        cmp #9
        bne @go_on
        lda #0
        sta song
@go_on:
        jsr init
@next:

        jsr play
        jsr vis_play
        
        jmp loop

NMI:
        pha
        tya
        pha
        txa
        pha

        lda #2
        sta $4014
        
        lda joy1
        sta joy1old

        ldy     #$01    ;
        sty     $4016   ; reset strobe
        dey
        sty     $4016   ; clear strobe
        sty     joy1    ; clears joystick data
        ldy     #$08    ; do all 8 buttons
read_buttons:
        lda     $4016   ; load button status
        and     #$01    ; only keep lowest bit
        lsr     a       ; transfer to carry flag
        rol     joy1
        dey
        bne     read_buttons
        
        pla
        tax
        pla
        tay
        pla

        rti
IRQ:
        rti

vis_init:
        lda $2002
        bpl vis_init
       lda #0
       sta $2006
       sta $2006
       sta $2007

@wait:
        lda $2002
        bpl @wait
        lda #$10
        sta $2006
        lda #0
        sta $2006
        lda #0
        sta $2007

        lda #$11
        sta $2006
        lda #0
        sta $2006
        ldy #64
        ldx #0
@load:        
        lda visplay_gfx,x
        sta $2007
        inx
        dey
        bne @load
@end_load:
        lda $2002
        bpl @end_load

        lda #$3F
        sta $2006
        lda #0
        sta $2006
        ldx #0
loadp:        
        lda default_pal + 12, x
        sta $2007
        inx
        cpx #4
        bne loadp
        
        lda #$3F
        sta $2006
        lda #$10
        sta $2006
        ldx #0
load_pal:
        lda default_pal, x
        sta $2007
        inx
        cpx #16
        bne load_pal
        
        lda #0
        sta $2006
        sta $2006
         
        lda #$20
        sta $204
        
        lda #$40
        sta $20C
        
        lda #$60
        sta $210

        lda #$80
        sta $208

        lda #1
        sta $20A
        lda #2
        sta $212
        
        lda #$12
        sta $205
        sta $209
        sta $20D
        sta $211
        rts

vis_play:
        ldx #0
        ldy #0
        jmp dot_coords ;SKIP    ; this will skip this part of the routine,
vp_tileloop:                    ; which takes the volume data and converts
       ;lda volume, x           ; it to a tile # for different sized dots
        ; maybe do an AND #16   ; I couldn't figure out where to get the
        lsr                     ; volume data from Silver Surfer
        lsr                     
        bne set_tile            ; if it's zero, set tile to 0 (blank)
        sta $209
        jmp skip
set_tile:
        lsr
        lsr
        clc
        adc #$10        ; the 4 dots are located at $10 in the pattern tables
        sta $209
skip:
        iny             ; inc index for saves to sprite ram
        iny             ;
        iny             ;
        iny             ;
        inx             ; inc index for load channel volumes
        cpx #4          ; if 4 channels are done, leave the loop
        bne vp_tileloop

dot_coords:
        ldy #0
        ldx #0
dots:
        lda $EC, x
        sta TempLo
        lda $ED, x
        sta TempHi
        clc
        ror TempHi
        ror TempLo
        clc
        ror TempHi
        ror TempLo
        lda TempLo
        eor #$FF                ; invert the data
        sta $207, y             ; store to sprite ram
        iny
        iny
        iny
        iny
        inx
        cpx #4
        bne dots
                
        rts

game:
        .byte "Silver Surfer", 0
author:
        .byte "Tim Follin and Geoff Follin", 0
copy:
        .byte "1990 Marvel", 0
info:
        .byte "Grafx routine by Memblers", 0
info2:
        .byte "NSF rip by TNSe1999", 0
press:
        .byte "Press start to change songs.", 0

visplay_gfx:
        .incbin "visplayr.gfx"          ; the tiles for the note dots (64 bytes)
font_gfx:
        .incbin "font.gfx"              ; the font
        .byte $7D          ; this byte acts as an EOF marker for the load routine, $7D is not used in the file of course.  Why $7D?  No particular reason..
default_pal:
        .incbin "visplay.pal"
